@@ -47,11 +47,11 @@ use std::io::{Read, Result, Seek, SeekFrom, Take};
 use std::io::Cursor;
 
 /// Parse a box out of a data buffer.
-pub fn read_box_header<T: ReadBytesExt>(src: &mut T) -> Option<BoxHeader> {
-    let tmp_size = src.read_u32::<BigEndian>().unwrap();
-    let name = src.read_u32::<BigEndian>().unwrap();
+pub fn read_box_header<T: ReadBytesExt>(src: &mut T) -> Result<BoxHeader> {
+    let tmp_size = try!(src.read_u32::<BigEndian>());
+    let name = try!(src.read_u32::<BigEndian>());
     let size = match tmp_size {
-        1 => src.read_u64::<BigEndian>().unwrap(),
+        1 => try!(src.read_u64::<BigEndian>()),
         _ => tmp_size as u64,
     };
     assert!(size >= 8);
@@ -63,10 +63,10 @@ pub fn read_box_header<T: ReadBytesExt>(src: &mut T) -> Option<BoxHeader> {
         _ => 4 + 4,
     };
     assert!(offset <= size);
-    Some(BoxHeader{
-        name: name,
-        size: size,
-        offset: offset,
+    Ok(BoxHeader{
+      name: name,
+      size: size,
+      offset: offset,
     })
 }
 
@@ -108,7 +108,13 @@ fn recurse<T: Read>(f: &mut T, h: &BoxHeader) {
         .collect();
     let mut content = Cursor::new(buf);
     loop {
-        read_box(&mut content);
+        match read_box(&mut content) {
+            Ok(_) => {},
+            Err(e) => {
+                println!("Error '{:?}' reading box", e.kind());
+                break;
+            },
+        }
     }
     println!("{} -- end", h);
 }
@@ -116,7 +122,7 @@ fn recurse<T: Read>(f: &mut T, h: &BoxHeader) {
 /// Read the contents of a box, including sub boxes.
 /// Right now it just prints the box value rather than
 /// returning anything.
-pub fn read_box<T: Read + Seek>(f: &mut T) {
+pub fn read_box<T: Read + Seek>(f: &mut T) -> Result<()> {
     read_box_header(f).and_then(|h| {
         match &(fourcc_to_string(h.name))[..] {
             "ftyp" => {
@@ -138,12 +144,37 @@ pub fn read_box<T: Read + Seek>(f: &mut T) {
             },
             _ => {
                 // Skip the contents of unknown chunks.
-                println!("{}", h);
+                println!("{} (skipped)", h);
                 skip_box_content(f, &h).unwrap();
             },
         };
-        Some(()) // and_then needs an Option to return.
+        Ok(()) // and_then needs a Result to return.
+    })
+}
+
+/// Entry point for C language callers.
+/// Take a buffer and call read_box() on it.
+#[no_mangle]
+pub unsafe extern fn read_box_from_buffer(buffer: *const u8, size: usize)
+  -> bool {
+    use std::slice;
+    use std::thread;
+
+    // Validate arguments from C.
+    if buffer.is_null() || size < 8 {
+        return false;
+    }
+
+    // Wrap the buffer we've been give in a slice.
+    let b = slice::from_raw_parts(buffer, size);
+    let mut c = Cursor::new(b);
+
+    // Parse in a subthread.
+    let task = thread::spawn(move || {
+        read_box(&mut c).unwrap();
     });
+    // Catch any panics.
+    task.join().is_ok()
 }
 
 
@@ -257,7 +288,7 @@ pub fn read_tkhd<T: ReadBytesExt>(src: &mut T, head: &BoxHeader)
 }
 
 /// Convert the iso box type or other 4-character value to a string.
-pub fn fourcc_to_string(name: u32) -> String {
+fn fourcc_to_string(name: u32) -> String {
     let u32_to_vec = |u| {
         vec!((u >> 24 & 0xffu32) as u8,
              (u >> 16 & 0xffu32) as u8,
@@ -292,12 +323,17 @@ impl fmt::Display for MovieHeaderBox {
     }
 }
 
+use std::u16;
 impl fmt::Display for TrackHeaderBox {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         let name = fourcc_to_string(self.name);
-        write!(f, "'{}' {} bytes id {} {}x{}",
-            name, self.size, self.track_id,
-            self.width, self.height)
+        // Dimensions are 16.16 fixed-point.
+        let base = u16::MAX as f64 + 1.0;
+        let width = (self.width as f64) / base;
+        let height = (self.height as f64) / base;
+        write!(f, "'{}' {} bytes duration {} id {} {}x{}",
+            name, self.size, self.duration, self.track_id,
+            width, height)
     }
 }
 
